package de.uni_marburg.powersort.data;

import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.converter.ConvertWith;
import org.junit.jupiter.params.converter.SimpleArgumentConverter;
import org.junit.jupiter.params.provider.CsvSource;

import java.util.Arrays;

import static org.junit.jupiter.api.Assertions.*;

class AscendingRunsTest {
    /**
     * https://stackoverflow.com/a/46850299/6334421
     */
    private static class IntArrayConverter extends SimpleArgumentConverter {
        @Override
        protected Object convert(Object source, Class<?> targetType) {
            if (source instanceof String s && int[].class.isAssignableFrom(targetType)) {
                if (s.isEmpty()) {
                    return new int[0];
                }
                String[] strings = s.split("\\s*\\|\\s*");
                return Arrays.stream(strings).mapToInt(Integer::valueOf).toArray();
            } else {
                throw new IllegalArgumentException("Conversion from " + source.getClass()
                        + " to " + targetType + " not supported.");
            }
        }

    }

    @ParameterizedTest
    @CsvSource({
            "0,1,-1,''",
            "1,1,-1,0",
            "1,2,-2,0|1",
            "2,2,-2,0|1|-2|-1",
            "2,4,-2,0|1|2|3|-2|-1|0|1",
    })
    void testAscendingRuns1(int numOfRuns, int runLength, int decreaseBetweenRuns, @ConvertWith(IntArrayConverter.class) int[] expected) {
        Integer[] actualIntegers = AscendingRuns.newAscendingRuns(numOfRuns, runLength, decreaseBetweenRuns).getCopy();
        int[] actual = Arrays.stream(actualIntegers).mapToInt(Integer::valueOf).toArray();
        assertArrayEquals(expected, actual);
    }

    @ParameterizedTest
    @CsvSource({
            "1,0,-1",
            "-1,1,-1",
    })
    void testAscendingRuns2(int numOfRuns, int runLength, int decreaseBetweenRuns) {
        assertThrows(IllegalArgumentException.class, () -> AscendingRuns.newAscendingRuns(numOfRuns, runLength, decreaseBetweenRuns));
    }
}
